//
// Adapted from an example by Rui Santos of randomnerdtutorials.com 
//     https://randomnerdtutorials.com/esp8266-web-server/
//
////////////////////////////////////////////////////////////////////

#include <ESP8266WiFi.h>
#include <FastLED.h>

#define NUM_LEDS 2
#define DATA_PIN 14

CRGB leds[NUM_LEDS];

// Replace with your network credentials
const char* ssid     = "XXXXXXXXXXXXXXXXXX";
const char* password = "XXXXXXXXXXXXXXXXXX";

// Set web server port number to 80
WiFiServer server(80);

// Variable to store the HTTP request
String header;

// Auxiliar variables to store the current output state
String LED_State = "RED";

// Current time
unsigned long currentTime = millis();
// Previous time
unsigned long previousTime = 0; 
// Define timeout time in milliseconds (example: 2000ms = 2s)
const long timeoutTime = 2000;

void setup() {
  Serial.begin(115200);
 
  // Connect to Wi-Fi network with SSID and password
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  // Print local IP address and start web server
  Serial.println("");
  Serial.println("WiFi connected.");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  server.begin();

  delay(500);
  FastLED.addLeds<WS2812B, DATA_PIN, GRB>(leds, NUM_LEDS);
  FastLED.setBrightness(  20 );
  leds[0] = CRGB::Red;
  leds[1] = CRGB::Red;
  FastLED.show();
}

void loop(){
  WiFiClient client = server.available();   // Listen for incoming clients

  if (client) {                             // If a new client connects,
    Serial.println("New Client.");          // print a message out in the serial port
    String currentLine = "";                // make a String to hold incoming data from the client
    currentTime = millis();
    previousTime = currentTime;
    while (client.connected() && currentTime - previousTime <= timeoutTime) { // loop while the client's connected
      currentTime = millis();         
      if (client.available()) {             // if there's bytes to read from the client,
        char c = client.read();             // read a byte, then
        Serial.write(c);                    // print it out the serial monitor
        header += c;
        if (c == '\n') {                    // if the byte is a newline character
          // if the current line is blank, you got two newline characters in a row.
          // that's the end of the client HTTP request, so send a response:
          if (currentLine.length() == 0) {
            // HTTP headers always start with a response code (e.g. HTTP/1.1 200 OK)
            // and a content-type so the client knows what's coming, then a blank line:
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();
            
            // sets LED colors
            if (header.indexOf("GET /red") >= 0) {
              LED_State = "RED";
              leds[0] = CRGB::Red;
              leds[1] = CRGB::Red;
              FastLED.show();
            } else if (header.indexOf("GET /green") >= 0) {
              LED_State = "GREEN";
              leds[0] = CRGB::Green;
              leds[1] = CRGB::Green;
              FastLED.show();
            } else if (header.indexOf("GET /blue") >= 0) {
              LED_State = "BLUE";
              leds[0] = CRGB::Blue;
              leds[1] = CRGB::Blue;
              FastLED.show();
            }
            
            // Display the HTML web page
            client.println("<!DOCTYPE html><html>");
            client.println("<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
            client.println("<link rel=\"icon\" href=\"data:,\">");
            // CSS to style the on/off buttons 
            // Feel free to change the background-color and font-size attributes to fit your preferences
            client.println("<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}");
            client.println(".button { background-color: #195B6A; border: none; color: white; padding: 16px 40px;");
            client.println("text-decoration: none; font-size: 30px; margin: 2px; cursor: pointer;}");
            client.println(".button2 {background-color: #77878A;}</style></head>");
            
            // Web Page Heading
            client.println("<body><h1>WiFi SAO Web Server</h1>");
            
            // Display current state 
            client.println("<p>LEDs are " + LED_State + "</p>");

            // Display current buttons 
            client.println("<p><a href=\"/red\"><button class=\"button\">RED</button></a></p>");
            client.println("<p><a href=\"/green\"><button class=\"button\">GREEN</button></a></p>");
            client.println("<p><a href=\"/blue\"><button class=\"button\">BLUE</button></a></p>");
            client.println("</body></html>");
            
            // The HTTP response ends with another blank line
            client.println();
            // Break out of the while loop
            break;
          } else { // if you got a newline, then clear currentLine
            currentLine = "";
          }
        } else if (c != '\r') {  // if you got anything else but a carriage return character,
          currentLine += c;      // add it to the end of the currentLine
        }
      }
    }
    // Clear the header variable
    header = "";
    // Close the connection
    client.stop();
    Serial.println("Client disconnected.");
    Serial.println("");
  }
}
